@@ -1,4 +1,4 @@
-from datetime import date, datetime
+from datetime import date
 from typing import Dict, List, Union
 
 from pydantic import AnyUrl, Field, root_validator, validator
@@ -38,13 +38,12 @@ from hsmodels.schemas.validators import (
 )
 
 
-class BaseAggregationMetadata(BaseMetadata):
+class BaseAggregationMetadataIn(BaseMetadata):
 
-    url: AnyUrl = Field(
-        title="Aggregation URL", description="An object containing the URL of the aggregation", allow_mutation=False
-    )
     title: str = Field(
-        title="Aggregation title", description="A string containing a descriptive title for the aggregation"
+        default=None,
+        title="Aggregation title",
+        description="A string containing a descriptive title for the aggregation",
     )
     subjects: List[str] = Field(
         default=[],
@@ -71,15 +70,9 @@ class BaseAggregationMetadata(BaseMetadata):
         title="Temporal coverage",
         description="An object containing the temporal coverage for a aggregation expressed as a date range",
     )
-    rights: Rights = Field(
-        default=None,
-        title="Rights statement",
-        description="An object containing information about the rights held in and over the aggregation and the license under which a aggregation is shared",
-    )
 
     _parse_additional_metadata = root_validator(pre=True, allow_reuse=True)(parse_additional_metadata)
     _parse_coverages = root_validator(pre=True, allow_reuse=True)(split_coverages)
-    _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
 
     _subjects_constraint = validator('subjects', allow_reuse=True)(subjects_constraint)
     _language_constraint = validator('language', allow_reuse=True)(language_constraint)
@@ -87,7 +80,7 @@ class BaseAggregationMetadata(BaseMetadata):
     _normalize_additional_metadata = root_validator(allow_reuse=True, pre=True)(normalize_additional_metadata)
 
 
-class GeographicRasterMetadata(BaseAggregationMetadata):
+class GeographicRasterMetadataIn(BaseAggregationMetadataIn):
     """
     A class used to represent the metadata associated with a geographic raster aggregation
 
@@ -101,14 +94,6 @@ class GeographicRasterMetadata(BaseAggregationMetadata):
 
         schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
-    type: AggregationType = Field(
-        const=True,
-        default=AggregationType.GeographicRasterAggregation,
-        title="Aggregation type",
-        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
-        allow_mutation=False,
-    )
-
     band_information: BandInformation = Field(
         title="Band information",
         description="An object containing information about the bands contained in the raster dataset",
@@ -125,7 +110,32 @@ class GeographicRasterMetadata(BaseAggregationMetadata):
     _parse_spatial_reference = validator("spatial_reference", pre=True, allow_reuse=True)(parse_spatial_reference)
 
 
-class GeographicFeatureMetadata(BaseAggregationMetadata):
+class GeographicRasterMetadata(GeographicRasterMetadataIn):
+
+    _type = AggregationType.GeographicRasterAggregation
+
+    type: AggregationType = Field(
+        const=True,
+        default=_type,
+        title="Aggregation type",
+        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
+        allow_mutation=False,
+    )
+
+    url: AnyUrl = Field(
+        title="Aggregation URL", description="An object containing the URL of the aggregation", allow_mutation=False
+    )
+
+    rights: Rights = Field(
+        default=None,
+        title="Rights statement",
+        description="An object containing information about the rights held in and over the aggregation and the license under which a aggregation is shared",
+    )
+
+    _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
+
+
+class GeographicFeatureMetadataIn(BaseAggregationMetadataIn):
     """
     A class used to represent the metadata associated with a geographic feature aggregation
 
@@ -139,14 +149,6 @@ class GeographicFeatureMetadata(BaseAggregationMetadata):
 
         schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
-    type: AggregationType = Field(
-        const=True,
-        default=AggregationType.GeographicFeatureAggregation,
-        title="Aggregation type",
-        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
-        allow_mutation=False,
-    )
-
     field_information: List[FieldInformation] = Field(
         default=[],
         title="Field information",
@@ -165,7 +167,30 @@ class GeographicFeatureMetadata(BaseAggregationMetadata):
     _parse_spatial_reference = validator("spatial_reference", pre=True, allow_reuse=True)(parse_spatial_reference)
 
 
-class MultidimensionalMetadata(BaseAggregationMetadata):
+class GeographicFeatureMetadata(GeographicFeatureMetadataIn):
+
+    type: AggregationType = Field(
+        const=True,
+        default=AggregationType.GeographicFeatureAggregation,
+        title="Aggregation type",
+        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
+        allow_mutation=False,
+    )
+
+    url: AnyUrl = Field(
+        title="Aggregation URL", description="An object containing the URL of the aggregation", allow_mutation=False
+    )
+
+    rights: Rights = Field(
+        default=None,
+        title="Rights statement",
+        description="An object containing information about the rights held in and over the aggregation and the license under which a aggregation is shared",
+    )
+
+    _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
+
+
+class MultidimensionalMetadataIn(BaseAggregationMetadataIn):
     """
     A class used to represent the metadata associated with a multidimensional space-time aggregation
 
@@ -179,14 +204,6 @@ class MultidimensionalMetadata(BaseAggregationMetadata):
 
         schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
-    type: AggregationType = Field(
-        const=True,
-        default=AggregationType.MultidimensionalAggregation,
-        title="Aggregation type",
-        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
-        allow_mutation=False,
-    )
-
     variables: List[Variable] = Field(
         default=[],
         title="Variables",
@@ -203,7 +220,30 @@ class MultidimensionalMetadata(BaseAggregationMetadata):
     )
 
 
-class ReferencedTimeSeriesMetadata(BaseAggregationMetadata):
+class MultidimensionalMetadata(MultidimensionalMetadataIn):
+
+    type: AggregationType = Field(
+        const=True,
+        default=AggregationType.MultidimensionalAggregation,
+        title="Aggregation type",
+        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
+        allow_mutation=False,
+    )
+
+    url: AnyUrl = Field(
+        title="Aggregation URL", description="An object containing the URL of the aggregation", allow_mutation=False
+    )
+
+    rights: Rights = Field(
+        default=None,
+        title="Rights statement",
+        description="An object containing information about the rights held in and over the aggregation and the license under which a aggregation is shared",
+    )
+
+    _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
+
+
+class ReferencedTimeSeriesMetadataIn(BaseAggregationMetadataIn):
     """
     A class used to represent the metadata associated with a referenced time series aggregation
 
@@ -217,6 +257,9 @@ class ReferencedTimeSeriesMetadata(BaseAggregationMetadata):
 
         schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
+
+class ReferencedTimeSeriesMetadata(ReferencedTimeSeriesMetadataIn):
+
     type: AggregationType = Field(
         const=True,
         default=AggregationType.ReferencedTimeSeriesAggregation,
@@ -225,8 +268,20 @@ class ReferencedTimeSeriesMetadata(BaseAggregationMetadata):
         allow_mutation=False,
     )
 
+    url: AnyUrl = Field(
+        title="Aggregation URL", description="An object containing the URL of the aggregation", allow_mutation=False
+    )
 
-class FileSetMetadata(BaseAggregationMetadata):
+    rights: Rights = Field(
+        default=None,
+        title="Rights statement",
+        description="An object containing information about the rights held in and over the aggregation and the license under which a aggregation is shared",
+    )
+
+    _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
+
+
+class FileSetMetadataIn(BaseAggregationMetadataIn):
     """
     A class used to represent the metadata associated with a file set aggregation
 
@@ -240,6 +295,9 @@ class FileSetMetadata(BaseAggregationMetadata):
 
         schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
+
+class FileSetMetadata(FileSetMetadataIn):
+
     type: AggregationType = Field(
         const=True,
         default=AggregationType.FileSetAggregation,
@@ -248,8 +306,20 @@ class FileSetMetadata(BaseAggregationMetadata):
         allow_mutation=False,
     )
 
+    url: AnyUrl = Field(
+        title="Aggregation URL", description="An object containing the URL of the aggregation", allow_mutation=False
+    )
 
-class SingleFileMetadata(BaseAggregationMetadata):
+    rights: Rights = Field(
+        default=None,
+        title="Rights statement",
+        description="An object containing information about the rights held in and over the aggregation and the license under which a aggregation is shared",
+    )
+
+    _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
+
+
+class SingleFileMetadataIn(BaseAggregationMetadataIn):
     """
     A class used to represent the metadata associated with a single file aggregation
 
@@ -262,6 +332,9 @@ class SingleFileMetadata(BaseAggregationMetadata):
 
         schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
+
+class SingleFileMetadata(SingleFileMetadataIn):
+
     type: AggregationType = Field(
         const=True,
         default=AggregationType.SingleFileAggregation,
@@ -270,8 +343,20 @@ class SingleFileMetadata(BaseAggregationMetadata):
         allow_mutation=False,
     )
 
+    url: AnyUrl = Field(
+        title="Aggregation URL", description="An object containing the URL of the aggregation", allow_mutation=False
+    )
 
-class TimeSeriesMetadata(BaseAggregationMetadata):
+    rights: Rights = Field(
+        default=None,
+        title="Rights statement",
+        description="An object containing information about the rights held in and over the aggregation and the license under which a aggregation is shared",
+    )
+
+    _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
+
+
+class TimeSeriesMetadataIn(BaseAggregationMetadataIn):
     """
     A class used to represent the metadata associated with a time series aggregation
 
@@ -287,14 +372,6 @@ class TimeSeriesMetadata(BaseAggregationMetadata):
 
         schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
-    type: AggregationType = Field(
-        const=True,
-        default=AggregationType.TimeSeriesAggregation,
-        title="Aggregation type",
-        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
-        allow_mutation=False,
-    )
-
     time_series_results: List[TimeSeriesResult] = Field(
         default=[],
         title="Time series results",
@@ -306,7 +383,30 @@ class TimeSeriesMetadata(BaseAggregationMetadata):
     _parse_abstract = root_validator(pre=True, allow_reuse=True)(parse_abstract)
 
 
-class ModelProgramMetadata(BaseAggregationMetadata):
+class TimeSeriesMetadata(TimeSeriesMetadataIn):
+
+    type: AggregationType = Field(
+        const=True,
+        default=AggregationType.TimeSeriesAggregation,
+        title="Aggregation type",
+        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
+        allow_mutation=False,
+    )
+
+    url: AnyUrl = Field(
+        title="Aggregation URL", description="An object containing the URL of the aggregation", allow_mutation=False
+    )
+
+    rights: Rights = Field(
+        default=None,
+        title="Rights statement",
+        description="An object containing information about the rights held in and over the aggregation and the license under which a aggregation is shared",
+    )
+
+    _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
+
+
+class ModelProgramMetadataIn(BaseAggregationMetadataIn):
     """
     A class used to represent the metadata associated with a model program aggregation
     """
@@ -316,14 +416,6 @@ class ModelProgramMetadata(BaseAggregationMetadata):
 
         schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
-    type: AggregationType = Field(
-        const=True,
-        default=AggregationType.ModelProgramAggregation,
-        title="Aggregation type",
-        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
-        allow_mutation=False,
-    )
-
     version: str = Field(
         default=None, title="Version", description="The software version or build number of the model", max_length=255
     )
@@ -378,7 +470,30 @@ class ModelProgramMetadata(BaseAggregationMetadata):
     _parse_file_types = root_validator(pre=True, allow_reuse=True)(parse_file_types)
 
 
-class ModelInstanceMetadata(BaseAggregationMetadata):
+class ModelProgramMetadata(ModelProgramMetadataIn):
+
+    type: AggregationType = Field(
+        const=True,
+        default=AggregationType.ModelProgramAggregation,
+        title="Aggregation type",
+        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
+        allow_mutation=False,
+    )
+
+    url: AnyUrl = Field(
+        title="Aggregation URL", description="An object containing the URL of the aggregation", allow_mutation=False
+    )
+
+    rights: Rights = Field(
+        default=None,
+        title="Rights statement",
+        description="An object containing information about the rights held in and over the aggregation and the license under which a aggregation is shared",
+    )
+
+    _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
+
+
+class ModelInstanceMetadataIn(BaseAggregationMetadataIn):
     """
     A class used to represent the metadata associated with a model instance aggregation
     """
@@ -388,14 +503,6 @@ class ModelInstanceMetadata(BaseAggregationMetadata):
 
         schema_config = {'read_only': ['type', 'url'], 'dictionary_field': ['additional_metadata']}
 
-    type: AggregationType = Field(
-        const=True,
-        default=AggregationType.ModelInstanceAggregation,
-        title="Aggregation type",
-        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
-        allow_mutation=False,
-    )
-
     includes_model_output: bool = Field(
         title="Includes Model Output",
         description="Indicates whether model output files are included in the aggregation",
@@ -418,3 +525,26 @@ class ModelInstanceMetadata(BaseAggregationMetadata):
         title="JSON metadata schema values URL",
         description="A URL to a JSON file containing the metadata values conforming to the JSON metadata schema for the related model program",
     )
+
+
+class ModelInstanceMetadata(ModelInstanceMetadataIn):
+
+    type: AggregationType = Field(
+        const=True,
+        default=AggregationType.ModelInstanceAggregation,
+        title="Aggregation type",
+        description="A string expressing the aggregation type from the list of HydroShare aggregation types",
+        allow_mutation=False,
+    )
+
+    url: AnyUrl = Field(
+        title="Aggregation URL", description="An object containing the URL of the aggregation", allow_mutation=False
+    )
+
+    rights: Rights = Field(
+        default=None,
+        title="Rights statement",
+        description="An object containing information about the rights held in and over the aggregation and the license under which a aggregation is shared",
+    )
+
+    _parse_url = root_validator(pre=True, allow_reuse=True)(parse_url)
